/***************************************************************************//**
*  \details    This driver will act as an etherbone slave on the host
*              and controls a wishbone master in the FPGA over PCIe
*
*  \author     Michael Reese
*
* *******************************************************************************/
#include<linux/kernel.h>
#include<linux/init.h>
#include<linux/module.h>
#include <linux/kdev_t.h>
#include <linux/err.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/cdev.h>

static int ebs_wbm_pcie_open(struct inode *inode, struct file *file)
{
        printk(KERN_INFO "%s\n", __func__);
        return 0;
}
static int ebs_wbm_pcie_release(struct inode *inode, struct file *file)
{
        printk(KERN_INFO "%s\n", __func__);
        return 0;
}
static ssize_t ebs_wbm_pcie_read(struct file *filp, char __user *buf, size_t len, loff_t *off)
{
        printk(KERN_INFO "%s\n", __func__);
        return 0;
}
static ssize_t ebs_wbm_pcie_write(struct file *filp, const char __user *buf, size_t len, loff_t *off)
{
        printk(KERN_INFO "%s\n", __func__);
        return len;
}

static struct file_operations fops =
{
    .owner      = THIS_MODULE,
    .read       = ebs_wbm_pcie_read,
    .write      = ebs_wbm_pcie_write,
    .open       = ebs_wbm_pcie_open,
    .release    = ebs_wbm_pcie_release,
};

static struct cdev cdev;
dev_t dev = 0;
static struct class *dev_class;
 
static int __init ebs_wbm_pcie_init(void)
{
    cdev_init(&cdev,&fops);
    if ((alloc_chrdev_region(&dev, 0, 1, "ebs_wbm_pcie_dev")) <0) {
        return -1;
    }
    if((cdev_add(&cdev,dev,1)) < 0){
        goto r_class;
    }
    dev_class = class_create("ebs_wbm_pcie_class");
    if(IS_ERR(dev_class)){
        goto r_class;
    }
    if(IS_ERR(device_create(dev_class,NULL,dev,NULL,"ebs_wbm_pcie_device"))){
        goto r_device;
    }

    printk(KERN_INFO "Simple kernel module inserted successfully\n");
    return 0;
 
r_device:
    class_destroy(dev_class);
r_class:
    unregister_chrdev_region(dev,1);
    return -1;
}

static void __exit ebs_wbm_pcie_exit(void)
{
    device_destroy(dev_class,dev);
    class_destroy(dev_class);
    unregister_chrdev_region(dev, 1);
    printk(KERN_INFO "Simple kernel module removed\n");
}
 
module_init(ebs_wbm_pcie_init);
module_exit(ebs_wbm_pcie_exit);
 
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Michael Reese <m.reese@gsi.de>");
MODULE_DESCRIPTION("Etherbone slave that controls a wishbone master module on the FPGA via PCIe");
MODULE_VERSION("1:0.0");
