#include "ebs.h"

#include<linux/module.h>


void eb_slave_reset(EbSlaveState *eb) {
	eb->state = EB_SLAVE_IDLE;
	eb->error_shift_register = 0;
}
void eb_slave_read_flags(EbSlaveState *eb) {
	eb->BCA = (eb->input[0] >> 7) & 0x1;
	eb->RCA = (eb->input[0] >> 6) & 0x1;
	eb->RFF = (eb->input[0] >> 5) & 0x1;
	eb->CYC = (eb->input[0] >> 3) & 0x1;
	eb->WCA = (eb->input[0] >> 2) & 0x1;
	eb->WFF = (eb->input[0] >> 1) & 0x1;
	eb->ByteEnable = eb->input[1];
	eb->WCount     = eb->input[2];
	eb->RCount     = eb->input[3];
}
void eb_slave_generate_read_response(EbSlaveState *eb) {
	eb->output[0] = (      0 << 7) // response BCA
	              | (      0 << 6) // response RCA
	              | (      0 << 5) // response RFF
	              | (eb->CYC << 3) // response CYC
	              | (eb->BCA << 2) // response WCA
	              | (eb->RFF << 1) // response WFF
	              ;
	eb->output[1] = eb->ByteEnable;
	eb->output[2] = eb->RCount;
	eb->output[3] = 0x0;
}
void eb_slave_set_output(EbSlaveState *eb, uint32_t value) {
	eb->output[0] = (value >> 24) & 0xff;
	eb->output[1] = (value >> 16) & 0xff;
	eb->output[2] = (value >>  8) & 0xff;
	eb->output[3] = (value >>  0) & 0xff;
}
uint32_t eb_slave_get_input(EbSlaveState *eb) {
	uint32_t result = (eb->input[0] << 24) | (eb->input[1] << 16) | (eb->input[2] << 8) | (eb->input[3] << 0);
	return result;
}
void eb_slave_process(EbSlaveState *eb, uint8_t *data) {
	int input;

	eb->input[0] = data[0];
	eb->input[1] = data[1];
	eb->input[2] = data[2];
	eb->input[3] = data[3];

	input = (eb->input[0] << 24) | (eb->input[1] << 16) | (eb->input[2] << 8) | (eb->input[3] << 0);
	switch(eb->state) {
	case EB_SLAVE_IDLE: 
		if (eb->input[0] != 0x4e || eb->input[1] != 0x6f) return;
		eb->output[0]=0x4e;
		eb->output[1]=0x6f;
		eb->output[2]=0x16;
		eb->output[3]=0x44;
		eb->state = EB_SLAVE_MAGIC_FOUND;
		break;
	case EB_SLAVE_MAGIC_FOUND:
		for (int i = 0; i < 4; ++i) { 
			eb->output[i] = eb->input[i]; 
		}
		eb->state = EB_SLAVE_EXPECT_RECORD;
		break;
	case EB_SLAVE_EXPECT_RECORD:
		eb_slave_read_flags(eb);
			// eb->BCA, eb->RCA, eb->RFF, eb->CYC, eb->WCA, eb->WFF, eb->ByteEnable, eb->WCount, eb->RCount);
		if (eb->WCount) {
			eb->state = EB_SLAVE_HANDLE_WRITES_BASE;
			eb_slave_set_output(eb, 0); // response for record header is 0x00000000 if WCount is > 0
		} else if (eb->RCount) {
			eb_slave_generate_read_response(eb);
			eb->state = EB_SLAVE_HANDLE_READS_BASE;
		} else {
			// no writes and no reads
			eb->state = EB_SLAVE_EXPECT_RECORD;
		}
		break;
	case EB_SLAVE_HANDLE_WRITES_BASE:
		eb->state = EB_SLAVE_HANDLE_WRITES;
		eb->BaseWriteAddr = eb_slave_get_input(eb);
		eb_slave_set_output(eb, 0); // response for record header is 0x00000000 if WCount is > 0
		break;
	case EB_SLAVE_HANDLE_WRITES:
		if (eb->WCount) {
			uint32_t value = eb_slave_get_input(eb);
			if (eb->WCA) {
				eb_slave_set_output(eb, 0); // response word for writes is 0x00000000
			} else {
				eb->hw_write(eb->BaseWriteAddr, value);
				eb_slave_set_output(eb, 0); // response word for writes is 0x00000000
			}
			if (!eb->WFF) eb->BaseWriteAddr += 4;
			--eb->WCount;
		}
		if (!eb->WCount) {
			if (eb->RCount) {
				// if there are reads, the response to the last write address is a write record header
				eb_slave_generate_read_response(eb);
				eb->state = EB_SLAVE_HANDLE_READS_BASE;
			} else {
				eb->state = EB_SLAVE_EXPECT_RECORD;
			}
		}
		break;
	case EB_SLAVE_HANDLE_READS_BASE:
		eb->BaseRetAddr = eb_slave_get_input(eb);
		for (int i = 0; i < 4; ++i) eb->output[i] = eb->input[i]; // Copy BaseRetAddr
		eb->state = EB_SLAVE_HANDLE_READS;
		break;
	case EB_SLAVE_HANDLE_READS:
		if (eb->RCount) {
			uint32_t addr = eb_slave_get_input(eb);

			if (eb->RCA) { // read from configuration space
				eb_slave_set_output(eb, eb->hw_cfg_read(addr));
			} else {
				eb_slave_set_output(eb, eb->hw_read(addr));
			}
			if (!eb->RFF) eb->BaseRetAddr += 4;
			--eb->RCount;
		}
		if (!eb->RCount) {
			eb->state = EB_SLAVE_EXPECT_RECORD;
		}
		break;
	}
	data[0] = eb->output[0];
	data[1] = eb->output[1];
	data[2] = eb->output[2];
	data[3] = eb->output[3];

}
